# __all__ = ('make_meshgrid','plot_contours','plot_classifier','plot_classifiers','plot_4_classifiers')

import numpy as np
import matplotlib.pyplot as plt

"""
==================================================
Plot different SVM classifiers in the iris dataset
==================================================

Comparison of different linear SVM classifiers on a 2D projection of the iris
dataset. We only consider the first 2 features of this dataset:

- Sepal length
- Sepal width

This example shows how to plot the decision surface for four SVM classifiers
with different kernels.

The linear models ``LinearSVC()`` and ``SVC(kernel='linear')`` yield slightly
different decision boundaries. This can be a consequence of the following
differences:

- ``LinearSVC`` minimizes the squared hinge loss while ``SVC`` minimizes the
  regular hinge loss.

- ``LinearSVC`` uses the One-vs-All (also known as One-vs-Rest) multiclass
  reduction while ``SVC`` uses the One-vs-One multiclass reduction.

Both linear models have linear decision boundaries (intersecting hyperplanes)
while the non-linear kernel models (polynomial or Gaussian RBF) have more
flexible non-linear decision boundaries with shapes that depend on the kind of
kernel and its parameters.

.. NOTE:: while plotting the decision function of classifiers for toy 2D
   datasets can help get an intuitive understanding of their respective
   expressive power, be aware that those intuitions don't always generalize to
   more realistic high-dimensional problems.

"""
# print(__doc__)


def make_meshgrid(x, y, h=.02):
    """Create a mesh of points to plot in

    Parameters
    ----------
    x: data to base x-axis meshgrid on
    y: data to base y-axis meshgrid on
    h: stepsize for meshgrid, optional

    Returns
    -------
    xx, yy : ndarray
    """
    x_min, x_max = x.min() - 1, x.max() + 1
    y_min, y_max = y.min() - 1, y.max() + 1
    xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))
    return xx, yy

def plot_contours(ax, clf, xx, yy, **params):
    """Plot the decision boundaries for a classifier.

    Parameters
    ----------
    ax: matplotlib axes object
    clf: a classifier
    xx: meshgrid ndarray
    yy: meshgrid ndarray
    params: dictionary of params to pass to contourf, optional
    """
    Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
    Z = Z.reshape(xx.shape)
    out = ax.contourf(xx, yy, Z, **params)
    return out

#%%

def plot_classifier(X, y, clf, ax=None, ticks=False, proba=False, lims=None): # assumes classifier "clf" is already fit
    X0, X1 = X[:, 0], X[:, 1]
    xx, yy = make_meshgrid(X0, X1)

    if ax is None:
        plt.figure()
        ax = plt.gca()
        show = True
    else:
        show = False

    # can abstract some of this into a higher-level function for learners to call
    cs = plot_contours(ax, clf, xx, yy, cmap=plt.cm.coolwarm, alpha=0.8, proba=proba)
    if proba:
        cbar = plt.colorbar(cs)
        cbar.ax.set_ylabel('probability of red $\Delta$ class', fontsize=20, rotation=270, labelpad=30)
        cbar.ax.tick_params(labelsize=14)
    #ax.scatter(X0, X1, c=y, cmap=plt.cm.coolwarm, s=30, edgecolors='k', linewidth=1)
    labels = np.unique(y)
    if len(labels) == 2:
        ax.scatter(X0[y==labels[0]], X1[y==labels[0]], cmap=plt.cm.coolwarm, s=60, c='b', marker='o', edgecolors='k')
        ax.scatter(X0[y==labels[1]], X1[y==labels[1]], cmap=plt.cm.coolwarm, s=60, c='r', marker='^', edgecolors='k')
    else:
        ax.scatter(X0, X1, c=y, cmap=plt.cm.coolwarm, s=50, edgecolors='k', linewidth=1)

    ax.set_xlim(xx.min(), xx.max())
    ax.set_ylim(yy.min(), yy.max())
#     ax.set_xlabel(data.feature_names[0])
#     ax.set_ylabel(data.feature_names[1])
    if ticks:
        ax.set_xticks(())
        ax.set_yticks(())
#     ax.set_title(title)
    if show:
        plt.show()
    else:
        return ax

def plot_4_classifiers(X, y, clfs):
    # Set-up 2x2 grid for plotting.
    fig, sub = plt.subplots(2, 2)
    plt.subplots_adjust(wspace=0.2, hspace=0.2)

    for clf, ax, title in zip(clfs, sub.flatten(), ("(1)","(2)","(3)","(4)")):
        # clf.fit(X, y)
        plot_classifier(X, y, clf, ax, ticks=True)
        ax.set_title(title)
    plt.show()

def plot_classifiers(X, y, clfs):
    titles = []
    for model in clfs:
      titles.append(model.__class__.__name__)

    # Set-up nx2 grid for plotting.
    nrows = int(len(clfs)/2) # assume len is even for now
    fig, sub = plt.subplots(nrows, 2)
    plt.subplots_adjust(wspace=0.2, hspace=0.2)

    for clf, ax, title in zip(clfs, sub.flatten(), titles):
        # clf.fit(X, y)
        plot_classifier(X, y, clf, ax, ticks=True)
        ax.set_title(title)
    plt.show()
    
#%%


# __all__ = ('dfchk')
import pandas as pd

#%%
# Standard quick checks
def dfchk(dframe, chkNull=True, info=True, head=True, shape=True, desc=False, valCnt = False): 
  """
  some basic and common checks on dataframes. 
  Args:
      dframe (Pandas.DataFrame): pandas dataframe
      chkNull (bool): check Null option, defaults to True
      info (bool): info option, defaults to True
      head (bool): head option, defaults to True
      shape (bool): shape option, defaults to True
      desc (bool): describe option, defaults to False
      valCnt (bool): value_count option, defaults to False
  Return: None
  """
  cnt = 1
  print('\ndataframe Basic Check function -')
  
  if (chkNull):
    print(f'\n{cnt}: Null values:')
    cnt+=1
    print( dframe.isnull().sum().sort_values(ascending=False) )
  
  if (info):
    try:
      print(f'\n{cnt}: info(): ')
      cnt+=1
      print(dframe.info())
    except: pass
    
  if (desc):
    print(f'\n{cnt}: describe(): ')
    cnt+=1
    print(dframe.describe())
  
  if (head):
    print(f'\n{cnt}: head() -- ')
    cnt+=1
    print(dframe.head())

  if (shape): 
    print(f'\n{cnt}: shape: ')
    cnt+=1
    print(dframe.shape)

  if (valCnt):
    print('\nValue Counts for each feature -')
    for colname in dframe.columns :
      print(f'\n{cnt}: {colname} value_counts(): ')
      print(dframe[colname].value_counts())
      cnt +=1

# examples:
# dfchk(df, desc=True)

#%%


# __all__ = ('api','dbcon','dfchk','boundary_plots')
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import mysql.connector
from mysql.connector import Error
import requests

# from . import helper
from .helper import * 
from . import boundary_plots
from .boundary_plots import * 



# __all__ = ('dfapi')
import pandas as pd
import numpy as np
import requests

def dfapi(tbname, ind_col_name = ""):
  """
  call to api endpoint on regression.fit database to access datasets
  Args:
      tbname (str): name of data table
      ind_col_name (str or int): numeric id of column or column name
  Return: pandas.Dataframe
  """
  df = None # set a global variable to store the dataframe
  apikey = 'K35wHcKuwXuhHTaz7zY42rCje'
  parameters = {"apikey": apikey, 'table': tbname }
  heads = { "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:60.0) Gecko/20100101 Firefox/60.0" } # server has ModSecurity check if it is a browser request. 
  js = {'error': 'Initialize' }

  try:
    response = requests.get("http://api.regression.fit/endpt.json", params=parameters, headers=heads)
    js = response.json()
  except BaseException as err: print(f'Error while connecting to regession.fit API. Please contact the administrator. {err=}, {type(err)=}')

  if ('error' in js) : 
    print(f'Error: {js["error"]} Please contact the administrator.') # The json object will have a key named "error" if not successful
    return df
  
  # json object seems okay at this point
  try: df = pd.DataFrame(js) 
  except ValueError: print(f'Value Error while converting json into dataframe. Please contact the administrator.')
  except BaseException as err: print(f'Error while converting json into dataframe. Please contact the administrator. {err=}, {type(err)=}')
  
  # df seems load okay at this point. Default values is object/string everywhere.
  # try to convert all possible ones to numeric
  df.replace({"NA": np.NaN, "na": np.NaN, "n/a": np.NaN, "N/A": np.NaN, "NaN": np.NaN, "nan": np.NaN }, inplace=True)
  for col in df.columns:
    try: df[col]=pd.to_numeric(df[col])
    except ValueError: pass
    except: pass

  # set index if given
  # if (ind_col_name and ind_col_name in df): df.set_index(ind_col_name, inplace=True)  # if given col_name exist, make it the index.
  try: df.set_index(ind_col_name, inplace=True)
  except ValueError: pass
  except TypeError: pass
  except: pass
  
  print(f'Dataframe from Regression.Fit API is loaded.')
  return df

# print("\nFunction api_rfit loaded. Ready to continue.")

